@@ -1,6 +1,6 @@
 #!/usr/bin/env ocaml
 (* libguestfs
- * Copyright (C) 2016 Red Hat Inc.
+ * Copyright (C) 2016-2018 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,12 @@
  * shell scripts located in libguestfs.git/builder/website.
  *)
 
+#load "str.cma";;
+#load "unix.cma";;
+#directory "../../ocaml";; (* use locally built guestfs *)
+(*#directory "+guestfs";; (* use globally installed guestfs *) *)
+#load "mlguestfs.cma";;
+
 open Printf
 
 let prog = "make-template"
@@ -42,22 +48,36 @@ let () =
      eprintf "%s: you must use `../../run ./make-template.ml ...' to run this script\n"
              prog;
      exit 1
+  );
+
+  (* Check we're not being run as root. *)
+  if Unix.geteuid () = 0 then (
+    eprintf "%s: don't run this script as root\n" prog;
+    exit 1
+  );
+  (* ... and that LIBVIRT_DEFAULT_URI=qemu:///system is NOT set,
+   * which is the same as above.
+   *)
+  let s = try Sys.getenv "LIBVIRT_DEFAULT_URI" with Not_found -> "" in
+  if s = "qemu:///system" then (
+    eprintf "%s: don't set LIBVIRT_DEFAULT_URI=qemu:///system\n" prog;
+    exit 1
   )
   ;;
 
-#load "str.cma";;
-#load "unix.cma";;
-#directory "../../ocaml";;
-#load "mlguestfs.cma";;
-
 type os =
   | CentOS of int * int         (* major, minor *)
   | RHEL of int * int
   | Debian of int * string      (* version, dist name like "wheezy" *)
   | Ubuntu of string * string
   | Fedora of int               (* version number *)
+  | FreeBSD of int * int        (* major, minor *)
 type arch = X86_64 | Aarch64 | Armv7 | I686 | PPC64 | PPC64le | S390X
 
+type boot_media =
+  | Location of string          (* virt-install --location (preferred) *)
+  | CDRom of string             (* downloaded CD-ROM *)
+
 let quote = Filename.quote
 let (//) = Filename.concat
 
@@ -74,34 +94,31 @@ let rec main () =
   (* For OSes which require a kickstart, this generates one.
    * For OSes which require a preseed file, this returns one (we
    * don't generate preseed files at the moment).
+   * For OSes which cannot be automated (FreeBSD), this returns None.
    *)
   let ks = make_kickstart_or_preseed os arch in
 
-  (* Find the virt-install --location for this OS. *)
-  let location = make_location os arch in
-
-  (* RHEL guests require alternate yum configuration pointing to
-   * Red Hat's internal servers.
+  (* Find the boot media.  Normally ‘virt-install --location’ but
+   * for FreeBSD it downloads the boot ISO.
    *)
-  let yum_conf =
-    match os with
-    | RHEL (major, minor) when major >= 5 ->
-       Some (make_rhel_yum_conf major minor arch)
-    | _ -> None in
+  let boot_media = make_boot_media os arch in
 
-  (* Choose a random temporary name for the libvirt domain.  We will
-   * also have to clean this up after running virt-install.
-   *)
+  (* Choose a random temporary name for the libvirt domain. *)
   let tmpname = sprintf "tmp-%s" (random8 ()) in
 
   (* Choose a random temporary disk name. *)
   let tmpout = sprintf "%s.img" tmpname in
 
+  (* Create the final output name (actually not quite final because
+   * we will xz-compress it).
+   *)
+  let output = filename_of_os os arch "" in
+
   (* Some architectures need EFI boot. *)
   let tmpefivars =
     match os, arch with
     | (Fedora _|RHEL _), Aarch64 ->
-       let vars = sprintf "%s.vars" tmpname in
+       let vars = Sys.getcwd () // sprintf "%s.vars" tmpname in
        let cmd =
          sprintf "cp /usr/share/edk2/aarch64/vars-template-pflash.raw %s"
                  (quote vars) in
@@ -111,20 +128,10 @@ let rec main () =
 
   (* Now construct the virt-install command. *)
   let vi = make_virt_install_command os arch ks tmpname tmpout tmpefivars
-                                     location virtual_size_gb in
-  (* Make sure that temporary guest and temporary files are removed
-   * if we exit for any reason.
-   *)
-  let cleanup_libvirt_guest () =
-    let cmd =
-      sprintf "virsh undefine%s %s >&/dev/null"
-              (if tmpefivars = None then "" else " --nvram")
-              (quote tmpname) in
-    ignore (Sys.command cmd)
-  in
+                                     boot_media virtual_size_gb in
+  (* Make sure that temporary files are removed if we exit for any reason. *)
   at_exit (
     fun () ->
-      cleanup_libvirt_guest ();
       (try Unix.unlink tmpout with _ -> ());
       (match tmpefivars with
        | Some (_, vars) -> (try Unix.unlink vars with _ -> ())
@@ -134,29 +141,60 @@ let rec main () =
   (* Print the virt-install command just before we run it, because
    * this is expected to be long-running.
    *)
-  Array.iter (
-    fun arg ->
-      if arg.[0] = '-' then printf "\\\n    %s " arg
-      else printf "%s " arg
-  ) vi;
-  printf "\n%!";
+  print_virt_install_command stdout vi;
+
+  (* Save the virt-install command to a file, for documentation. *)
+  let chan = open_out (filename_of_os os arch ".virt-install-cmd") in
+  fprintf chan "# This is the virt-install command which was used to create\n";
+  fprintf chan "# the virt-builder template '%s'\n" (string_of_os os arch);
+  fprintf chan "# NB: This file is generated for documentation purposes ONLY!\n";
+  fprintf chan "# This script was never run, and is not intended to be run.\n";
+  fprintf chan "\n";
+  print_virt_install_command chan vi;
+  close_out chan;
+
+  (* Print the virt-install notes for OSes which cannot be automated
+   * fully.  (These are different from the ‘notes=’ section in the
+   * index fragment).
+   *)
+  print_install_notes os;
+  printf "\n\n%!";
 
   (* Run the virt-install command. *)
   let pid = Unix.fork () in
   if pid = 0 then Unix.execvp "virt-install" vi;
   let _, pstat = Unix.waitpid [] pid in
   check_process_status_for_errors pstat;
-  cleanup_libvirt_guest ();
+  (* If there were NVRAM variables, move them to the final name and
+   * compress them.  Doing this operation later means the cleanup of
+   * the guest will remove them as well (because of --nvram).
+   *)
+  let nvram =
+    match tmpefivars with
+    | Some (_, vars) ->
+       let f = sprintf "%s-nvram" output in
+       let cmd = sprintf "mv %s %s" (quote vars) (quote f) in
+       if Sys.command cmd <> 0 then exit 1;
+       let cmd = sprintf "xz -f --best %s" (quote f) in
+       if Sys.command cmd <> 0 then exit 1;
+       Some (f ^ ".xz")
+    | None -> None in
+
   ignore (Sys.command "sync");
 
   (* Run virt-filesystems, simply to display the filesystems in the image. *)
   let cmd = sprintf "virt-filesystems -a %s --all --long -h" (quote tmpout) in
   if Sys.command cmd <> 0 then exit 1;
 
+  (* Some guests are special flowers that need post-installation
+   * filesystem changes.
+   *)
+  let postinstall = make_postinstall os arch in
+
   (* Get the root filesystem.  If the root filesystem is LVM then
    * get the partition containing it.
    *)
-  let g = open_guest tmpout in
+  let g = open_guest ~mount:(postinstall <> None) tmpout in
   let roots = g#inspect_get_roots () in
   let expandfs, lvexpandfs =
     let rootfs = g#canonical_device_name roots.(0) in
@@ -173,18 +211,9 @@ let rec main () =
       | [] | _::_::_ -> assert false
     ) in
 
-  (* Some guests are special flowers. *)
-  (match os with
-   | Debian _ ->
-      (* Remove apt proxy configuration (thanks: Daniel Miranda). *)
-      g#rm_f "/etc/apt/apt.conf";
-      g#touch "/etc/apt/apt.conf";
-   | _ -> ()
-  );
-  (match yum_conf with
-   | Some yum_conf ->
-      g#write "/etc/yum.repos.d/download.devel.redhat.com.repo" yum_conf;
+  (match postinstall with
    | None -> ()
+   | Some f -> f g
   );
 
   g#shutdown ();
@@ -203,18 +232,17 @@ let rec main () =
    | _ -> ()
   );
 
-  (* Sysprep.  Relabel SELinux-using guests. *)
-  printf "Sysprepping ...\n%!";
-  let cmd =
-    sprintf "virt-sysprep --quiet -a %s%s"
-            (quote tmpout)
-            (if is_selinux_os os then " --selinux-relabel" else "") in
-  if Sys.command cmd <> 0 then exit 1;
-
-  (* Create the final output name (actually not quite final because
-   * we will xz-compress it).
-   *)
-  let output = filename_of_os os arch "" in
+  (match os with
+   | FreeBSD _ -> () (* virt-sysprep doesn't work on FreeBSD. *)
+   | _ ->
+      (* Sysprep.  Relabel SELinux-using guests. *)
+      printf "Sysprepping ...\n%!";
+      let cmd =
+        sprintf "virt-sysprep --quiet -a %s%s"
+                (quote tmpout)
+                (if is_selinux_os os then " --selinux-relabel" else "") in
+      if Sys.command cmd <> 0 then exit 1
+  );
 
   (* Sparsify and copy to output name. *)
   printf "Sparsifying ...\n%!";
@@ -230,20 +258,6 @@ let rec main () =
   if Sys.command cmd <> 0 then exit 1;
   let output = output ^ ".xz" in
 
-  (* If there were NVRAM variables, move them to the final name and
-   * compress them too.
-   *)
-  let nvram =
-    match tmpefivars with
-    | Some (_, vars) ->
-       let f = sprintf "%s-nvram" output in
-       let cmd = sprintf "mv %s %s-nvram" (quote vars) (quote f) in
-       if Sys.command cmd <> 0 then exit 1;
-       let cmd = sprintf "xz -f --best %s-nvram" (quote f) in
-       if Sys.command cmd <> 0 then exit 1;
-       Unix.unlink vars;
-       Some (f ^ ".xz")
-    | None -> None in
   printf "Template completed: %s\n%!" output;
 
   (* Construct the index fragment, but don't create this for the private
@@ -316,11 +330,13 @@ and os_of_string os ver =
   | "debian", "6" -> Debian (6, "squeeze")
   | "debian", "7" -> Debian (7, "wheezy")
   | "debian", "8" -> Debian (8, "jessie")
+  | "debian", "9" -> Debian (9, "stretch")
   | "ubuntu", "10.04" -> Ubuntu (ver, "lucid")
   | "ubuntu", "12.04" -> Ubuntu (ver, "precise")
   | "ubuntu", "14.04" -> Ubuntu (ver, "trusty")
   | "ubuntu", "16.04" -> Ubuntu (ver, "xenial")
   | "fedora", ver -> Fedora (int_of_string ver)
+  | "freebsd", ver -> let maj, min = parse_major_minor ver in FreeBSD (maj, min)
   | _ ->
      eprintf "%s: unknown or unsupported OS (%s, %s)\n" prog os ver; exit 1
 
@@ -355,6 +371,15 @@ and string_of_arch = function
   | PPC64le -> "ppc64le"
   | S390X -> "s390x"
 
+and debian_arch_of_arch = function
+  | X86_64 -> "amd64"
+  | Aarch64 -> "arm64"
+  | Armv7 -> "armhf"
+  | I686 -> "i386"
+  | PPC64 -> "ppc64"
+  | PPC64le -> "ppc64el"
+  | S390X -> "s390x"
+
 and filename_of_os os arch ext =
   match os with
   | Fedora ver ->
@@ -372,6 +397,9 @@ and filename_of_os os arch ext =
   | Ubuntu (ver, _) ->
      if arch = X86_64 then sprintf "ubuntu-%s%s" ver ext
      else sprintf "ubuntu-%s-%s%s" ver (string_of_arch arch) ext
+  | FreeBSD (major, minor) ->
+     if arch = X86_64 then sprintf "freebsd-%d.%d%s" major minor ext
+     else sprintf "freebsd-%d.%d-%s%s" major minor (string_of_arch arch) ext
 
 and string_of_os os arch = filename_of_os os arch ""
 
@@ -382,10 +410,12 @@ and string_of_os_noarch = function
   | RHEL (major, minor) -> sprintf "rhel-%d.%d" major minor
   | Debian (ver, _) -> sprintf "debian-%d" ver
   | Ubuntu (ver, _) -> sprintf "ubuntu-%s" ver
+  | FreeBSD (major, minor) -> sprintf "freebsd-%d.%d" major minor
 
 and is_selinux_os = function
   | RHEL _ | CentOS _ | Fedora _ -> true
-  | Debian _ | Ubuntu _ -> false
+  | Debian _ | Ubuntu _
+  | FreeBSD _ -> false
 
 and get_virtual_size_gb os arch = 6
 
@@ -394,11 +424,14 @@ and make_kickstart_or_preseed os arch =
   (* Kickstart. *)
   | Fedora _ | CentOS _ | RHEL _ ->
      let ks_filename = filename_of_os os arch ".ks" in
-     make_kickstart_common ks_filename os arch
+     Some (make_kickstart_common ks_filename os arch)
 
   (* Preseed. *)
-  | Debian _ -> copy_preseed_to_temporary "debian.preseed"
-  | Ubuntu _ -> copy_preseed_to_temporary "ubuntu.preseed"
+  | Debian _ -> Some (copy_preseed_to_temporary "debian.preseed")
+  | Ubuntu _ -> Some (copy_preseed_to_temporary "ubuntu.preseed")
+
+  (* Not automated. *)
+  | FreeBSD _ -> None
 
 and make_kickstart_common ks_filename os arch =
   let buf = Buffer.create 4096 in
@@ -458,7 +491,7 @@ part /     --fstype=%s   --size=1024 --grow --asprimary
    | CentOS _ | RHEL _ | Fedora _ ->
       bpf "\
 zerombr
-clearpart --all --initlabel
+clearpart --all --initlabel --disklabel=gpt
 autopart --type=plain
 ";
    | _ -> assert false (* cannot happen, see caller *)
@@ -577,13 +610,14 @@ and copy_preseed_to_temporary source =
   if Sys.command cmd <> 0 then exit 1;
   f
 
-and make_location os arch =
+and make_boot_media os arch =
   match os, arch with
   | CentOS (major, _), Aarch64 ->
      (* XXX This always points to the latest CentOS, so
       * effectively the minor number is always ignored.
       *)
-     sprintf "http://mirror.centos.org/altarch/%d/os/aarch64/" major
+     Location (sprintf "http://mirror.centos.org/altarch/%d/os/aarch64/"
+                       major)
 
   | CentOS (major, _), X86_64 ->
      (* For 6.x we rebuild this every time there is a new 6.x release, and bump
@@ -591,89 +625,129 @@ and make_location os arch =
       * For 7.x this always points to the latest CentOS, so
       * effectively the minor number is always ignored.
       *)
-     sprintf "http://mirror.centos.org/centos-7/%d/os/x86_64/" major
+     Location (sprintf "http://mirror.centos.org/centos-7/%d/os/x86_64/"
+                       major)
 
-  | Debian (_, dist), X86_64 ->
-     sprintf "http://deb.debian.org/debian/dists/%s/main/installer-amd64"
-             dist
-
-  | Fedora ver, Aarch64 ->
-     sprintf "https://download.fedoraproject.org/pub/fedora-secondary/releases/%d/Server/aarch64/os/" ver
+  | Debian (_, dist), arch ->
+     Location (sprintf "http://deb.debian.org/debian/dists/%s/main/installer-%s"
+                       dist (debian_arch_of_arch arch))
 
+  (* Fedora primary architectures. *)
   | Fedora ver, Armv7 ->
-     sprintf "http://mirror.bytemark.co.uk/fedora/linux/releases/%d/Server/armhfp/os/" ver
+     Location (sprintf "http://mirror.bytemark.co.uk/fedora/linux/releases/%d/Server/armhfp/os/" ver)
 
   | Fedora ver, X86_64 when ver < 21 ->
-     sprintf "http://mirror.bytemark.co.uk/fedora/linux/releases/%d/Fedora/x86_64/os/" ver
+     Location (sprintf "http://mirror.bytemark.co.uk/fedora/linux/releases/%d/Fedora/x86_64/os/" ver)
 
   | Fedora ver, X86_64 ->
-     sprintf "http://mirror.bytemark.co.uk/fedora/linux/releases/%d/Server/x86_64/os/" ver
+     Location (sprintf "http://mirror.bytemark.co.uk/fedora/linux/releases/%d/Server/x86_64/os/" ver)
 
-  | Fedora ver, I686 when ver < 21 ->
-     sprintf "http://mirror.bytemark.co.uk/fedora/linux/releases/%d/Fedora/i386/os/" ver
+  | Fedora ver, Aarch64 ->
+     Location (sprintf "http://mirror.bytemark.co.uk/fedora/linux/releases/%d/Server/aarch64/os/" ver)
 
+  (* Fedora secondary architectures.
+   * By using dl.fedoraproject.org we avoid randomly using mirrors
+   * which might have incomplete copies.
+   *)
   | Fedora ver, I686 ->
-     sprintf "http://mirror.bytemark.co.uk/fedora/linux/releases/%d/Server/i386/os/" ver
+     Location (sprintf "https://dl.fedoraproject.org/pub/fedora-secondary/releases/%d/Server/i386/os/" ver)
 
   | Fedora ver, PPC64 ->
-     sprintf "https://download.fedoraproject.org/pub/fedora-secondary/releases/%d/Server/ppc64/os/" ver
+     Location (sprintf "https://dl.fedoraproject.org/pub/fedora-secondary/releases/%d/Server/ppc64/os/" ver)
 
   | Fedora ver, PPC64le ->
-     sprintf "https://download.fedoraproject.org/pub/fedora-secondary/releases/%d/Server/ppc64le/os/" ver
+     Location (sprintf "https://dl.fedoraproject.org/pub/fedora-secondary/releases/%d/Server/ppc64le/os/" ver)
 
   | Fedora ver, S390X ->
-     sprintf "https://download.fedoraproject.org/pub/fedora-secondary/releases/%d/Server/s390x/os/" ver
+     Location (sprintf "https://dl.fedoraproject.org/pub/fedora-secondary/releases/%d/Server/s390x/os/" ver)
 
   | RHEL (3, minor), X86_64 ->
-     sprintf "http://download.devel.redhat.com/released/RHEL-3/U%d/AS/x86_64/tree" minor
+     Location (sprintf "http://download.devel.redhat.com/released/RHEL-3/U%d/AS/x86_64/tree" minor)
 
   | RHEL (4, minor), X86_64 ->
-     sprintf "http://download.devel.redhat.com/released/RHEL-4/U%d/AS/x86_64/tree" minor
+     Location (sprintf "http://download.devel.redhat.com/released/RHEL-4/U%d/AS/x86_64/tree" minor)
 
   | RHEL (5, minor), I686 ->
-     sprintf "http://download.devel.redhat.com/released/RHEL-5-Server/U%d/i386/os" minor
+     Location (sprintf "http://download.devel.redhat.com/released/RHEL-5-Server/U%d/i386/os" minor)
 
   | RHEL (5, minor), X86_64 ->
-     sprintf "http://download.devel.redhat.com/released/RHEL-5-Server/U%d/x86_64/os" minor
+     Location (sprintf "http://download.devel.redhat.com/released/RHEL-5-Server/U%d/x86_64/os" minor)
 
   | RHEL (6, minor), I686 ->
-     sprintf "http://download.devel.redhat.com/released/RHEL-6/6.%d/Server/i386/os" minor
+     Location (sprintf "http://download.devel.redhat.com/released/RHEL-6/6.%d/Server/i386/os" minor)
 
   | RHEL (6, minor), X86_64 ->
-     sprintf "http://download.devel.redhat.com/released/RHEL-6/6.%d/Server/x86_64/os" minor
+     Location (sprintf "http://download.devel.redhat.com/released/RHEL-6/6.%d/Server/x86_64/os" minor)
 
   | RHEL (7, minor), X86_64 ->
-     sprintf "http://download.devel.redhat.com/released/RHEL-7/7.%d/Server/x86_64/os" minor
+     Location (sprintf "http://download.devel.redhat.com/released/RHEL-7/7.%d/Server/x86_64/os" minor)
 
   | RHEL (7, minor), Aarch64 ->
-     sprintf "http://download.eng.bos.redhat.com/released/RHEL-7/7.%d/Server/aarch64/os" minor
+     Location (sprintf "http://download.eng.bos.redhat.com/released/RHEL-7/7.%d/Server/aarch64/os" minor)
 
   | RHEL (7, minor), PPC64 ->
-     sprintf "http://download.devel.redhat.com/released/RHEL-7/7.%d/Server/ppc64/os" minor
+     Location (sprintf "http://download.devel.redhat.com/released/RHEL-7/7.%d/Server/ppc64/os" minor)
 
   | RHEL (7, minor), PPC64le ->
-     sprintf "http://download.devel.redhat.com/released/RHEL-7/7.%d/Server/ppc64le/os" minor
+     Location (sprintf "http://download.devel.redhat.com/released/RHEL-7/7.%d/Server/ppc64le/os" minor)
 
   | Ubuntu (_, dist), X86_64 ->
-     sprintf "http://archive.ubuntu.com/ubuntu/dists/%s/main/installer-amd64"
-             dist
+     Location (sprintf "http://archive.ubuntu.com/ubuntu/dists/%s/main/installer-amd64" dist)
 
   | Ubuntu (_, dist), PPC64le ->
-     sprintf "http://ports.ubuntu.com/ubuntu-ports/dists/%s/main/installer-ppc64el" dist
+     Location (sprintf "http://ports.ubuntu.com/ubuntu-ports/dists/%s/main/installer-ppc64el" dist)
+
+  | FreeBSD (major, minor), X86_64 ->
+     let iso = sprintf "FreeBSD-%d.%d-RELEASE-amd64-disc1.iso"
+                       major minor in
+     let iso_xz = sprintf "ftp://ftp.freebsd.org/pub/FreeBSD/releases/amd64/amd64/ISO-IMAGES/%d.%d/%s.xz"
+                       major minor iso in
+     let cmd = sprintf "wget -nc %s" (quote iso_xz) in
+     if Sys.command cmd <> 0 then exit 1;
+     let cmd = sprintf "unxz -f --keep %s.xz" iso in
+     if Sys.command cmd <> 0 then exit 1;
+     CDRom iso
 
   | _ ->
      eprintf "%s: don't know how to calculate the --location for this OS and architecture\n" prog;
      exit 1
 
+and print_install_notes = function
+  | Ubuntu _ ->
+     printf "\
+Some preseed functions are not automated.  You may need to hit [Return]
+a few times during the install.\n"
+
+  | FreeBSD _ ->
+     printf "\
+The FreeBSD install is not automated.  Select all defaults, except:
+
+ - root password:  builder
+ - timezone:       UTC
+ - do not add any user accounts\n"
+
+  | _ -> ()
+
+(* If the install is not automated and we need a graphical console. *)
+and needs_graphics = function
+  | CentOS _ | RHEL _ | Debian _ | Ubuntu _ | Fedora _ -> false
+  | FreeBSD _ -> true
+
 (* NB: Arguments do not need to be quoted, because we pass them
  * directly to exec(2).
  *)
-and make_virt_install_command os arch ks tmpname tmpout tmpefivars location
-                              virtual_size_gb =
+and make_virt_install_command os arch ks tmpname tmpout tmpefivars
+                              boot_media virtual_size_gb =
   let args = ref [] in
   let add arg = args := arg :: !args in
 
   add "virt-install";
+
+  (* This ensures the libvirt domain will be automatically deleted
+   * when virt-install exits.
+   *)
+  add "--transient";
+
   add (sprintf "--name=%s" tmpname);
 
   (*add "--print-xml";*)
@@ -691,20 +765,19 @@ and make_virt_install_command os arch ks tmpname tmpout tmpefivars location
    | PPC64 ->
       add "--arch=ppc64";
       add "--machine=pseries";
-      add "--cpu=POWER7";
+      add "--cpu=power7";
       add "--vcpus=1"
    | PPC64le ->
       add "--arch=ppc64le";
       add "--machine=pseries";
-      add "--cpu=POWER8";
+      add "--cpu=power8";
       add "--vcpus=1"
    | arch ->
       add (sprintf "--arch=%s" (string_of_arch arch));
       add "--vcpus=1"
   );
 
-  add "--os-type=linux";
-  add (sprintf "--os-variant=%s" (os_variant_of_os os arch));
+  add (sprintf "--os-variant=%s" (os_variant_of_os ~for_fedora:true os arch));
 
   (match tmpefivars with
    | Some (code, vars) ->
@@ -714,62 +787,102 @@ and make_virt_install_command os arch ks tmpname tmpout tmpefivars location
    | _ -> ()
   );
 
-  add (sprintf "--initrd-inject=%s" ks);
-  let os_extra =
-    match os with
-    | Debian _ | Ubuntu _ -> "auto"
-    | Fedora _ | RHEL _ | CentOS _ ->
-       sprintf "ks=file:/%s" (Filename.basename ks) in
-  let proxy =
-    let p = try Some (Sys.getenv "http_proxy") with Not_found -> None in
-    match p with
-    | None -> ""
-    | Some p ->
-       match os with
-       | Fedora _ | RHEL _ | CentOS _ -> "proxy=" ^ p
-       | Debian _ | Ubuntu _ -> "mirror/http/proxy=" ^ p in
+  (match ks with
+   | None -> ()
+   | Some ks ->
+      add (sprintf "--initrd-inject=%s" ks);
 
-  add (sprintf "--extra-args=%s %s %s" (* sic: does NOT need to be quoted *)
-               os_extra proxy (kernel_cmdline_of_os os arch));
+      let os_extra =
+        match os with
+        | Debian _ | Ubuntu _ -> "auto"
+        | Fedora _ | RHEL _ | CentOS _ ->
+           sprintf "ks=file:/%s" (Filename.basename ks)
+        | FreeBSD _ -> assert false in
+      let proxy =
+        let p = try Some (Sys.getenv "http_proxy") with Not_found -> None in
+        match p with
+        | None ->
+           (match os with
+            | Fedora _ | RHEL _ | CentOS _ | Ubuntu _ -> ""
+            | Debian _ -> "mirror/http/proxy="
+            | FreeBSD _ -> assert false
+           )
+        | Some p ->
+           match os with
+           | Fedora _ | RHEL _ | CentOS _ -> "proxy=" ^ p
+           | Debian _ | Ubuntu _ -> "mirror/http/proxy=" ^ p
+           | FreeBSD _ -> assert false in
+
+      add (sprintf "--extra-args=%s %s %s" (* sic: does NOT need to be quoted *)
+                   os_extra proxy (kernel_cmdline_of_os os arch));
+  );
 
   add (sprintf "--disk=%s,size=%d,format=raw"
                (Sys.getcwd () // tmpout) virtual_size_gb);
 
+  (match boot_media with
+   | Location location -> add (sprintf "--location=%s" location)
+   | CDRom iso -> add (sprintf "--cdrom=%s" iso)
+  );
+
   add "--serial=pty";
-  add (sprintf "--location=%s" location);
-  add "--nographics";
-  add "--noreboot";
+  if not (needs_graphics os) then add "--nographics";
 
   (* Return the command line (list of arguments). *)
   Array.of_list (List.rev !args)
 
-and os_variant_of_os os arch =
-  match os, arch with
-  (* This case is needed to workaround a bug in virt-install:
-   * https://bugzilla.redhat.com/show_bug.cgi?id=1399083
-   *)
-  | Fedora _, (PPC64|PPC64le) -> "fedora22"
-  | Fedora ver, _ when ver <= 23 ->
-     sprintf "fedora%d" ver
-  | Fedora _, _ -> "fedora23" (* max version known in Fedora 24 *)
-  | CentOS (major, minor), _ when (major, minor) <= (7,0) ->
-     sprintf "centos%d.%d" major minor
-  | CentOS _, _ -> "centos7.0" (* max version known in Fedora 24 *)
-  | RHEL (major, minor), _ when (major, minor) <= (7,2) ->
-     sprintf "rhel%d.%d" major minor
-  | RHEL _, _ -> "rhel7.2" (* max version known in Fedora 24 *)
-  | Debian (ver, _), _ -> sprintf "debian%d" ver
-  | Ubuntu (ver, _), _ -> sprintf "ubuntu%s" ver
+and print_virt_install_command chan vi =
+  Array.iter (
+    fun arg ->
+      if arg.[0] = '-' then fprintf chan "\\\n    %s " (quote arg)
+      else fprintf chan "%s " (quote arg)
+  ) vi;
+  fprintf chan "\n\n%!"
 
-(* Same as the above, but we print the "true" os-variant, even
- * if osinfo itself might not support it yet.
+(* The optional [?for_fedora] flag means that we only return
+ * libosinfo data as currently supported by the latest version of
+ * Fedora.
+ *
+ * This is because if you try to use [virt-install --os-variant=...]
+ * with an os-variant which the host doesn't support, it won't work,
+ * and I currently use Fedora, so whatever is supported there matters.
  *)
-and true_os_variant_of_os = function
-  | Fedora ver -> sprintf "fedora%d" ver
-  | CentOS (major, minor) -> sprintf "centos%d.%d" major minor
-  | RHEL (major, minor) -> sprintf "rhel%d.%d" major minor
-  | Debian (ver, _) -> sprintf "debian%d" ver
-  | Ubuntu (ver, _) -> sprintf "ubuntu%s" ver
+and os_variant_of_os ?(for_fedora = false) os arch =
+  if not for_fedora then (
+    match os with
+    | Fedora ver -> sprintf "fedora%d" ver
+    | CentOS (major, minor) -> sprintf "centos%d.%d" major minor
+    | RHEL (major, minor) -> sprintf "rhel%d.%d" major minor
+    | Debian (ver, _) -> sprintf "debian%d" ver
+    | Ubuntu (ver, _) -> sprintf "ubuntu%s" ver
+    | FreeBSD (major, minor) -> sprintf "freebsd%d.%d" major minor
+  )
+  else (
+    match os, arch with
+    (* This special case for Fedora/ppc64{,le} is needed to work
+     * around a bug in virt-install:
+     * https://bugzilla.redhat.com/show_bug.cgi?id=1399083
+     *)
+    | Fedora _, (PPC64|PPC64le) -> "fedora22"
+    | Fedora ver, _ when ver <= 23 ->
+       sprintf "fedora%d" ver
+    | Fedora _, _ -> "fedora26" (* max version known in Fedora 28 *)
+
+    | CentOS (major, minor), _ when (major, minor) <= (7,0) ->
+       sprintf "centos%d.%d" major minor
+    | CentOS _, _ -> "centos7.0" (* max version known in Fedora 24 *)
+
+    | RHEL (major, minor), _ when (major, minor) <= (7,2) ->
+       sprintf "rhel%d.%d" major minor
+    | RHEL _, _ -> "rhel7.2" (* max version known in Fedora 24 *)
+
+    | Debian (ver, _), _ when ver <= 8 -> sprintf "debian%d" ver
+    | Debian _, _ -> "debian8" (* max version known in Fedora 26 *)
+
+    | Ubuntu (ver, _), _ -> sprintf "ubuntu%s" ver
+
+    | FreeBSD (major, minor), _ -> sprintf "freebsd%d.%d" major minor
+  )
 
 and kernel_cmdline_of_os os arch =
   match os, arch with
@@ -786,6 +899,29 @@ and kernel_cmdline_of_os os arch =
   | (RHEL _|CentOS _), PPC64
   | (RHEL _|CentOS _), PPC64le ->
      "console=tty0 console=ttyS0,115200 rd_NO_PLYMOUTH"
+  | FreeBSD _, (PPC64|PPC64le) -> assert false
+
+and make_postinstall os arch =
+  match os with
+  | Debian _ | Ubuntu _ ->
+     Some (
+       fun g ->
+         (* Remove apt proxy configuration (thanks: Daniel Miranda). *)
+         g#rm_f "/etc/apt/apt.conf";
+         g#touch "/etc/apt/apt.conf"
+     )
+
+  | RHEL (major, minor) when major >= 5 ->
+     Some (
+       fun g ->
+         (* RHEL guests require alternate yum configuration pointing to
+          * Red Hat's internal servers.
+          *)
+         let yum_conf = make_rhel_yum_conf major minor arch in
+         g#write "/etc/yum.repos.d/download.devel.redhat.com.repo" yum_conf
+     )
+
+  | RHEL _ | Fedora _ | CentOS _ | FreeBSD _ -> None
 
 and make_rhel_yum_conf major minor arch =
   let buf = Buffer.create 4096 in
@@ -873,7 +1009,7 @@ and make_index_fragment os arch index_fragment output nvram revision
 
   fpf "[%s]\n" (string_of_os_noarch os);
   fpf "name=%s\n" (long_name_of_os os arch);
-  fpf "osinfo=%s\n" (true_os_variant_of_os os);
+  fpf "osinfo=%s\n" (os_variant_of_os os arch);
   fpf "arch=%s\n" (string_of_arch arch);
   fpf "file=%s\n" output;
   (match revision with
@@ -928,6 +1064,10 @@ and long_name_of_os os arch =
      sprintf "Ubuntu %s (%s)" ver dist
   | Ubuntu (ver, dist), arch ->
      sprintf "Ubuntu %s (%s) (%s)" ver dist (string_of_arch arch)
+  | FreeBSD (major, minor), X86_64 ->
+     sprintf "FreeBSD %d.%d" major minor
+  | FreeBSD (major, minor), arch ->
+     sprintf "FreeBSD %d.%d (%s)" major minor (string_of_arch arch)
 
 and notes_of_os os arch nvram =
   let args = ref [] in
@@ -949,6 +1089,8 @@ and notes_of_os os arch nvram =
    | RHEL _ -> assert false (* cannot happen, see caller *)
    | Ubuntu _ ->
       add "This is a minimal Ubuntu install."
+   | FreeBSD _ ->
+      add "This is an all-default FreeBSD install."
   );
   add "";
 
@@ -980,16 +1122,10 @@ and notes_of_os os arch nvram =
    | CentOS (6, _) ->
       add "‘virt-builder centos-6’ will always install the latest 6.x release.";
       add ""
-   | Debian (8, _) ->
+   | Debian ((8|9), _) ->
       reconfigure_ssh_host_keys_debian ();
-      fix_serial_console_debian ();
    | Debian _ ->
       add "This image is so very minimal that it only includes an ssh server";
-      add "and no virtual consoles.  To enable virtual consoles use this";
-      add "virt-builder option:";
-      add "";
-      add "    --edit '/etc/inittab: s,^#([1-9].*respawn.*/sbin/getty.*),$1,'\n";
-      add "";
       reconfigure_ssh_host_keys_debian ();
    | Ubuntu ("16.04", _) ->
       builder_account_warning ();
@@ -1049,7 +1185,7 @@ and sha512sum_of_file filename =
 
 and size_of_file filename = (Unix.stat filename).Unix.st_size
 
-and open_guest filename =
+and open_guest ?(mount = false) filename =
   let g = new Guestfs.guestfs () in
   g#add_drive_opts ~format:"raw" filename;
   g#launch ();
@@ -1059,12 +1195,14 @@ and open_guest filename =
     eprintf "%s: cannot inspect this guest - this may mean guest installation failed\n" prog;
     exit 1
   );
-  let root = roots.(0) in
 
-  let mps = g#inspect_get_mountpoints root in
-  let cmp (a,_) (b,_) = compare (String.length a) (String.length b) in
-  let mps = List.sort cmp mps in
-  List.iter (fun (mp, dev) -> g#mount dev mp) mps;
+  if mount then (
+    let root = roots.(0) in
+    let mps = g#inspect_get_mountpoints root in
+    let cmp (a,_) (b,_) = compare (String.length a) (String.length b) in
+    let mps = List.sort cmp mps in
+    List.iter (fun (mp, dev) -> g#mount dev mp) mps
+  );
 
   g
 
