@@ -1,5 +1,5 @@
 /* libguestfs - the guestfsd daemon
- * Copyright (C) 2009-2017 Red Hat Inc.
+ * Copyright (C) 2009-2018 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,16 +33,6 @@
 #include "daemon.h"
 #include "actions.h"
 
-GUESTFSD_EXT_CMD(str_printenv, printenv);
-GUESTFSD_EXT_CMD(str_ldd, ldd);
-GUESTFSD_EXT_CMD(str_ls, ls);
-GUESTFSD_EXT_CMD(str_find, find);
-GUESTFSD_EXT_CMD(str_xargs, xargs);
-GUESTFSD_EXT_CMD(str_file, file);
-GUESTFSD_EXT_CMD(str_grep, grep);
-GUESTFSD_EXT_CMD(str_gawk, gawk);
-GUESTFSD_EXT_CMD(str_sh, sh);
-
 /* This command exposes debugging information, internals and
  * status.  There is no comprehensive documentation for this
  * command.  You have to look at the source code in this file
@@ -301,7 +291,7 @@ debug_env (const char *subcmd, size_t argc, char *const *const argv)
   char *out;
   CLEANUP_FREE char *err = NULL;
 
-  r = command (&out, &err, str_printenv, NULL);
+  r = command (&out, &err, "printenv", NULL);
   if (r == -1) {
     reply_with_error ("printenv: %s", err);
     free (out);
@@ -380,16 +370,13 @@ debug_binaries (const char *subcmd, size_t argc, char *const *const argv)
   int r;
   char *out;
   CLEANUP_FREE char *err = NULL;
-  char cmd[256];
+  const char *cmd =
+    "find / -xdev -type f -executable "
+    "| xargs file -i "
+    "| grep application/x-executable "
+    "| gawk -F: '{print $1}'";
 
-  snprintf (cmd, sizeof (cmd),
-            "%s / -xdev -type f -executable "
-            "| %s %s -i "
-            "| %s application/x-executable "
-            "| %s -F: '{print $1}'",
-            str_find, str_xargs, str_file, str_grep, str_gawk);
-
-  r = command (&out, &err, str_sh, "-c", cmd, NULL);
+  r = command (&out, &err, "sh", "-c", cmd, NULL);
   if (r == -1) {
     reply_with_error ("find: %s", err);
     free (out);
@@ -420,7 +407,7 @@ debug_ldd (const char *subcmd, size_t argc, char *const *const argv)
    * Also 'ldd' randomly sends messages to stderr and errors to stdout
    * depending on the phase of the moon.
    */
-  r = command (&out, &err, str_ldd, "-r", argv[0], NULL);
+  r = command (&out, &err, "ldd", "-r", argv[0], NULL);
   if (r == -1) {
     reply_with_error ("ldd: %s: %s", argv[0], err);
     free (out);
@@ -444,7 +431,7 @@ debug_ldd (const char *subcmd, size_t argc, char *const *const argv)
 static char *
 debug_ls (const char *subcmd, size_t argc, char *const *const argv)
 {
-  const size_t len = count_strings (argv);
+  const size_t len = guestfs_int_count_strings (argv);
   CLEANUP_FREE const char **cargv = NULL;
   size_t i;
   int r;
@@ -457,7 +444,7 @@ debug_ls (const char *subcmd, size_t argc, char *const *const argv)
     return NULL;
   }
 
-  cargv[0] = str_ls;
+  cargv[0] = "ls";
   cargv[1] = "-a";
   for (i = 0; i < len; ++i)
     cargv[2+i] = argv[i];
@@ -477,7 +464,7 @@ debug_ls (const char *subcmd, size_t argc, char *const *const argv)
 static char *
 debug_ll (const char *subcmd, size_t argc, char *const *const argv)
 {
-  const size_t len = count_strings (argv);
+  const size_t len = guestfs_int_count_strings (argv);
   CLEANUP_FREE const char **cargv = NULL;
   size_t i;
   int r;
@@ -490,7 +477,7 @@ debug_ll (const char *subcmd, size_t argc, char *const *const argv)
     return NULL;
   }
 
-  cargv[0] = str_ls;
+  cargv[0] = "ls";
   cargv[1] = "-la";
   for (i = 0; i < len; ++i)
     cargv[2+i] = argv[i];
