@@ -1,5 +1,5 @@
 /* guestfish - guest filesystem shell
- * Copyright (C) 2011-2017 Red Hat Inc.
+ * Copyright (C) 2011-2018 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -294,7 +294,7 @@ run_list_events (const char *cmd, size_t argc, char *argv[])
 {
   if (argc != 0) {
     fprintf (stderr,
-             _("use 'list-events' to list event handlers\n"));
+             _("use ‘list-events’ to list event handlers\n"));
     return -1;
   }
 
