@@ -1,5 +1,5 @@
 /* libguestfs
- * Copyright (C) 2009-2017 Red Hat Inc.
+ * Copyright (C) 2009-2018 Red Hat Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -54,6 +54,7 @@
 #include "guestfs.h"
 #include "guestfs-internal.h"
 #include "guestfs-internal-actions.h"
+#include "structs-cleanups.h"
 
 #if HAVE_FUSE
 
@@ -1046,7 +1047,7 @@ guestfs_impl_mount_local (guestfs_h *g, const char *localmountpoint,
 
   /* Set g->localmountpoint in the handle. */
   gl_lock_lock (mount_local_lock);
-  g->localmountpoint = localmountpoint;
+  g->localmountpoint = safe_strdup (g, localmountpoint);
   gl_lock_unlock (mount_local_lock);
 
   return 0;
@@ -1073,7 +1074,7 @@ guestfs_impl_mount_local_run (guestfs_h *g)
   r = guestfs_exists (g, "/");
   guestfs_pop_error_handler (g);
   if (r == -1) {
-    error (g, _("you must call 'guestfs_mount' first to mount a filesystem on '/'.\nNote: '%s' is still mounted.  Use 'guestunmount %s' to clean up."),
+    error (g, _("you must call ‘guestfs_mount’ first to mount a filesystem on '/'.\nNote: ‘%s’ is still mounted.  Use ‘guestunmount %s’ to clean up."),
            g->localmountpoint, g->localmountpoint);
     return -1;
   }
@@ -1089,6 +1090,7 @@ guestfs_impl_mount_local_run (guestfs_h *g)
 
   guestfs_int_free_fuse (g);
   gl_lock_lock (mount_local_lock);
+  free (g->localmountpoint);
   g->localmountpoint = NULL;
   gl_lock_unlock (mount_local_lock);
 
@@ -1147,7 +1149,8 @@ guestfs_impl_umount_local (guestfs_h *g,
     return -1;
   if (WIFEXITED (r) && WEXITSTATUS (r) == EXIT_SUCCESS)
     /* External fusermount succeeded.  Note that the original thread
-     * is responsible for setting g->localmountpoint to NULL.
+     * is responsible for freeing memory and setting
+     * g->localmountpoint to NULL.
      */
     return 0;
 
